import QtQuick 1.1
import "quickshow"

Presentation {
    id: root

    enableMouse: false

    ImageSlide {
        source: "resources/cover.png"
        title: "Qt Components & Qt Styles in Qt 5"
        authors: "Daker F. Pinheiro\nThiago Lacerda"
        date: "June 2012"
    }

    RegularSlide {
        title: "Daker Pinheiro (dakerfp)"

        content: [
            "UFPE - Brazil",
            "GSoC Plasma Components",
            "Software Engineer at INdT",
            "MeeGo Components",
            "Mobile and Desktop QtQuick"
        ]

        Image {
            anchors {
                right: parent.right
                verticalCenter: parent.verticalCenter
            }
            scale: 0.4
            source: "resources/recife.jpg"
        }
    }

    RegularSlide {
        title: "Thiago Lacerda (lacerda)"

        content: [
            "UFPE - Brazil",
            "Researcher at UFPE - Software Engineer at LG Electronics",
            "Software Engineer at INdT",
            "Qt-Components for MeeGo and other projects using Qt/QML"
        ]
    }

    // RegularSlide {
    //     title: "Agenda"

    //     content: [
    //         "History of Qt Components", // MeeGo, Plasma
    //         "Qt 5",
    //         "Research",
    //         "QtCS and future"
    //     ]
    // }

    DividerSlide {
        centeredText: "Once upon a time..."
    }

    RegularSlide {
        title: "History of Qt Components"

        // Widgets are everywhere

        Image {
            anchors.centerIn: parent
            source: "resources/widgets.png"
            scale: 0.8
            smooth: true
        }
    }

    RegularSlide {
        title: "History of Qt Components"

        content: [
            "QtWidgets",
            "Graphics View",
            "Qt 4.7: QML & QtQuick"
        ]
    }

    RegularSlide {
        title: "History of Qt Components"

        content: [
            "QML everywhere",
            "Lack of widgets",
            "No standard set of components for QML",
            "QML is fun!"
        ]
    }

    RegularSlide {
        title: "History of Qt Components"

        content: [
            "MeeGo Qt Components",
            "Symbian Qt Components",
            "Common API: https://bugreports.qt-project.org//browse/QTCOMPONENTS-200"
        ]
    }

    RegularSlide {
        title: "Desktop Components"

        Image {
            anchors.centerIn: parent
            source: "resources/desktop-components.png"
        }
    }

    RegularSlide {
        // GSoC
        title: "Plasma Components"

        Image {
            anchors.centerIn: parent
            source: "resources/plasma.jpg"
        }
    }

    DividerSlide {
        centeredText: "Qt5"
    }

    RegularSlide {
        title: "Qt5"

        content: [
            "QtQuick 2",
            "Scenegraph as Canvas"
        ]
    }

    RegularSlide {
        title: "Qt5"

        content: [
            "OpenGL",
            "QQuickPaintedItem",
        ]
    }

    DividerSlide {
        centeredText: "Our Research"
    }

    RegularSlide {
        title: "Motivation"

        content: [
            "Desktop Components vs. Qt5",
            "Independency of QtWidgets",
            "Native components",
            "Customizable components",
            "Lots of people claiming for that =)"
        ]
    }

    RegularSlide {
        title: "Goals"

        content: [
            "Rethink about styling in QML",
            "Rethink about components itself",
            "Think about the API"
        ]
    }

    RegularSlide {
        title: "Achievements"

        content: [
            "Easy styling for apps",
            "Disruption with QtWidgets / Proof of Concept", // facilidade de tirar o codigo que importa do QStyle
            "Modularization",
            "API for styling"
        ]
    }

    RegularSlide {
        title: "Easy styling"

        CodeSection {
            width: parent.width
            anchors.left: parent.left
            text: '
FocusScope {
    id: root

    readonly property bool pressed: mouseArea.pressed
    readonly property bool hover: mouseArea.containsMouse
    property alias hoverEnabled: mouseArea.hoverEnabled
    property string text
    property alias style: loader.sourceComponent
    property alias styleUrl: loader.source

    signal clicked()

    Loader {
        id: loader
        property Item button: root
        anchors.fill: parent
    }

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        ...
    }
}
'
        }
    }

    RegularSlide {
        title: "Easy styling for apps"

        CodeSection {
            id: btnStlyeCode
            anchors.left: parent.left
            text: '// CustomButtonStyle.qml

Rectangle {
    id: rect
    color: button.pressed ? "white"
                          : "gray"
    Text {
        anchors.centerIn: parent
        text: button.text
        // button reference
        // is injected when
        // the componenet
        // loads the style
    }
}
'
        }

        CodeSection {
            anchors {
                right: parent.right
                left: btnStlyeCode.right
                leftMargin: 20
            }
            text: 'Button {
    text: "Ok"
    style: CustomButtonStyle { }
    onClicked: {
        // action
    }
}
'
        }
    }

    RegularSlide {
        title: "Easy styling for apps"

        CodeSection {
            anchors.left: parent.left
            width: parent.width * 0.75
            text: 'ApplicationStyle {
    id: appStyle
    buttonStyle: CustomButtonStyle {
        background: "myButton.png"
        pressedBackground: "myPressedButton.png"
    }
    // all buttons will have CustomButtonStyle
    // look & feel. All other widgets on the
    // application will have the native
    // look & feel by default.
}

Button { } // Button with custom style
'
        }
    }

    RegularSlide {
        title: "Disruption of QtWidgets / Proof of Concept"

        content: [
            "Extract the code that matters",
            "Synchronization with QStyle code", // esses dois últimos são os problemas de extrair codigo do qstyle
            "Huge effort"
        ]
    }

    RegularSlide {
        title: "Modularization"

        Image {
            anchors.centerIn: parent
            source: "resources/combo.png"
        }
    }

    RegularSlide {
        title: "Modularization"
        CodeSection {
            anchors.left: parent.left
            width: parent.width
            text: "
class Q_UISTYLE_EXPORT QUiCommonButton : public QUiAbstractCommonButton
{
    Q_OBJECT
public:
    QUiCommonButton(QQuickItem *parent = 0);

    void paint(QPainter *painter);
    virtual void paintBackground(QPainter *painter) = 0;
    virtual void paintLabel(QPainter *painter);
    virtual void paintIcon(QPainter *painter);
    virtual void paintFocusRect(QPainter *painter, QRect rect);

    . . .
"
        }
    }


    RegularSlide {
        title: "API for Styling"

        content: [
            "Easy to create a new Custom Style (C++ or QML)",
            "Just need to watch the API's properties and create your desired behavior" // acho que aqui, ou num próximo slide, podia entrar um trecho de código mostrando a API em cpp que fizemos pro botão, por exemplo, drawBackgroung, drawFocusRect... pra mostrar que é só derivar dessa classe e implementar o que quer mudar
        ]
    }

    RegularSlide {
        content: [
            "Code: git clone git://code.openbossa.org/projects/qtquickstyles.git",
            "Wiki: http://wiki.qtlabs.org.br/projects/qtquickstyles/Main"
        ]
    }

    DividerSlide {
        centeredText: "Qt Contributors Summit 2012 & Future"
    }

    RegularSlide {
        title: "Qt Components"

        content: [
            "Widgets",
            "Layout",
            "Styling",
            "Window & Popups"
        ]
    }

    RegularSlide {
        title: "Qt Components"

        content: [
            "API is what matters",
            "Minimalistic API in Philosophy"
        ]
    }

    RegularSlide {
        title: "Qt Components"

        Image {
            anchors {
                right: parent.horizontalCenter
                verticalCenter: parent.verticalCenter
                rightMargin: 20
            }
            source: "resources/touch.png"
        }

        Image {
            anchors {
                left: parent.horizontalCenter
                verticalCenter: parent.verticalCenter
                leftMargin: 20
            }
            source: "resources/mouse-key.jpg"
        }
    }


    RegularSlide {
        title: "Qt Components Touch"

        content: [
            "Reference Style Implementation",
            "KISS",
            "Performance",
            "Fancy look and feel",
            "Platform free"
        ]
    }

    RegularSlide {
        title: "Qt Components Desktop"

        content: [
            "Desktop oriented components",
            "Native look and feel",
            "Delegate styling approach",
            "Natives Style - QStyle (!)"
        ]
    }

    RegularSlide {
        title: "Qt Components Desktop"

        content: [
            "Merging the best ideas around components",
            "Plasma Components ?",
            "Plasma Active Components ?"
        ]
    }

    RegularSlide {
        title: "Qt Components"

        content: [
            "Wiki page: http://qt-project.org/wiki/Qt_Components",
            "IRC: #qt-components at irc.freenod.net",
            "Mailing list: http://lists.qt-project.org/mailman/listinfo/qt-components"
        ]
    }

    RegularSlide {
        centeredText: "Thanks =)"
    }
}
